/*
 * HttpServlet.java -- Abstract base class for all http servlets
 *
 * Copyright (c) 1998, 1999 by Free Software Foundation, Inc.
 * Written by Paul Siegmann (pauls@euronet.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation, version 2. (see COPYING.LIB)
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA
 */
package javax.servlet.http;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.Enumeration;

import javax.servlet.GenericServlet;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

/**
 * The mother-of-all-HttpServlets.
 * Every normal http servlet extends this class and overrides either the doGet
 * or doPost methods (or both).
 * The server calls service. Service in its turn calls doGet, doPost, whatever,
 * depending on the client's request.
 *
 * @version 19990221
 * @author Paul Siegmann (pauls@euronet.nl)
 */
public abstract class HttpServlet
	extends GenericServlet
	implements Serializable 
{
	private String optionString = null;

	// whether doHead calls a subclassed doGet implementation or uses
	// its own internal implementation
	private boolean doHeadCallsDoGet = false;

	private final String basisClassName = "javax.servlet.http.HttpServlet";
	private Method[] methodList; // filled in constructor, used in doOptions

	public HttpServlet() {
		try {
			methodList = this.getClass().getMethods();
			for(int i = 0; i < methodList.length; i++) {
				if("doGet".equals(methodList[i].getName())
				 &&(! basisClassName.equals(methodList[i].getDeclaringClass().getName()))) {
					doHeadCallsDoGet = true;
					break;
				}
			}
		} catch(SecurityException e) {
		}
	}


	/**
	 * This method is called on a "DELETE" request.
	 * The default implementation is that 
	 * HttpServletResponse.SC_BAD_REQUEST (error nr: 400)
	 * and the message "Method \"DELETE\" is not supported by this servlet"
	 * is returned to the client.
	 *
	 * @exception ServletException if an Servlet Exception occurs
	 * @exception IOException if an IOException occurs
	 */
	protected void doDelete(HttpServletRequest request,
				HttpServletResponse response) throws ServletException, IOException {
		response.sendError(HttpServletResponse.SC_BAD_REQUEST,"Method \"DELETE\" is not supported by this servlet");
	}


	/**
	 * This method is called on a "GET" request.
	 * The default implementation is that 
	 * HttpServletResponse.SC_BAD_REQUEST (error nr: 400)
	 * and the message "Method \"GET\" is not supported by this servlet"
	 * is returned to the client.
	 *
	 * @exception ServletException if an Servlet Exception occurs
	 * @exception IOException if an IOException occurs
	 */
	protected void doGet(HttpServletRequest request,
				HttpServletResponse response) throws ServletException, IOException {
		response.sendError(HttpServletResponse.SC_BAD_REQUEST,"Method \"GET\" is not supported by this servlet");
	}





	/**
	 * This method is called on a "OPTIONS" request.
	 * It tells the client which methods are supported by the servlet.<BR>
	 * This comes down to an implementation where HEAD, TRACE and OPTIONS
	 * are supported by default because this class contains default
	 * implementations of them. GET, POST, DELETE and PUT are added
	 * to this list if the subclass has its own implementation of
	 * the doGet, doPost, doDelete or doPut method respectively.
	 * Note:<BR>
	 * This implementation is probably not the most efficient one,
	 * but the whole OPTIONS thing is intended for debugging
	 * purposes, and this implementation does the job.
	 *
	 * @exception ServletException if an Servlet Exception occurs
	 * @exception IOException if an IOException occurs
	 */
	protected void doOptions(HttpServletRequest request,
					HttpServletResponse response) throws ServletException, IOException {
		if(optionString == null) {
			StringBuffer result = new StringBuffer("");
			for(int i = 0; i < methodList.length; i++) {
				if("doGet".equals(methodList[i].getName())
				 &&(! basisClassName.equals(methodList[i].getDeclaringClass().getName()))) {
						result.append("GET, ");
				} else if("doDelete".equals(methodList[i].getName())
					&&(! basisClassName.equals(methodList[i].getDeclaringClass().getName()))) {
						result.append("DELETE, ");
				} else if("doPut".equals(methodList[i].getName())
					&&(! basisClassName.equals(methodList[i].getDeclaringClass().getName()))) {
						result.append("PUT, ");
				} else if("doPost".equals(methodList[i].getName())
					&&(! basisClassName.equals(methodList[i].getDeclaringClass().getName()))) {
						result.append("POST, ");
				}
			}
			result.append("HEAD, TRACE, OPTIONS");
			optionString = result.toString();
		}
		response.setHeader("Allow",optionString);
	}


	/**
	 * This method is called on a "POST" request.
	 * The default implementation is that 
	 * HttpServletResponse.SC_BAD_REQUEST (error nr: 400)
	 * and the message "Method \"POST\" is not supported by this servlet"
	 * is returned to the client.
	 *
	 * @exception ServletException if an Servlet Exception occurs
	 * @exception IOException if an IOException occurs
	 */
	protected void doPost(HttpServletRequest request,
				HttpServletResponse response) throws ServletException, IOException {
		response.sendError(HttpServletResponse.SC_BAD_REQUEST,"Method \"POST\" is not supported by this servlet");
	}


	/**
	 * This method is called on a "PUT" request.
	 * The default implementation is that 
	 * HttpServletResponse.SC_BAD_REQUEST (error nr: 400)
	 * and the message "Method \"PUT\" is not supported by this servlet"
	 * is returned to the client.
	 *
	 * @exception ServletException if an Servlet Exception occurs
	 * @exception IOException if an IOException occurs
	 */
	protected void doPut(HttpServletRequest request,
				HttpServletResponse response) throws ServletException, IOException {
		response.sendError(HttpServletResponse.SC_BAD_REQUEST,"Method \"PUT\" is not supported by this servlet");
	}


	/**
	 * This method is called on a "TRACE" request.
	 * This method is for debugging purposes.<BR>
	 * When a client makes a TRACE request the following is returned:<BR>
	 * content type = "message/http"<BR>
	 * message size: &lt;size of the complete message&gt;<BR>
	 * first line of the message : TRACE &lt;requested uri&gt;
	 * &lt;protocol&gt;<BR>
	 * on the following lines all the request header names and values
	 *
	 * @exception ServletException if an Servlet Exception occurs
	 * @exception IOException if an IOException occurs
	 */
	protected void doTrace(HttpServletRequest request,
				HttpServletResponse response) throws ServletException, IOException {
		StringBuffer result = new StringBuffer("\r\n");

		result.append("TRACE " + request.getRequestURI() + " " + request.getProtocol() + "\r\n");
		String headerName;
  		for(Enumeration e = request.getHeaderNames(); e.hasMoreElements();) {
			headerName = (String)e.nextElement();
			result.append(headerName + ": " + request.getHeader(headerName) + "\r\n");
		}

		response.setContentType("message/http");
		response.setContentLength(result.length());
		PrintWriter out = response.getWriter();
		out.print(result.toString());
		out.close();
	}


	/**
	 * Returns the time the requested uri was last modified in seconds since 1 january 1970.
	 */
	protected long getLastModified(HttpServletRequest request) {
		return -1;
	}


	/**
	 * This method looks whether the request is a POST, GET, etc method,
	 * and then calls the appropriate doPost, doGet, whatever method.<BR>
	 * If the request method is something it can't handle it sends
	 * a HttpServletResponse.SC_BAD_REQUEST error through the response.
	 *
	 * @exception ServletException an error has occured
	 * @exception IOException an error has occured
	 */
	protected void service(HttpServletRequest request,
				HttpServletResponse response) throws ServletException, IOException {
		if(request.getMethod().equals("GET") ) {
			doGet(request,response);
		} else if (request.getMethod().equals("HEAD") ) {
			doHead(request,response);
		} else if (request.getMethod().equals("POST") ) {
			doPost(request,response);
		} else if (request.getMethod().equals("DELETE") ) {
			doDelete(request,response);
		} else if (request.getMethod().equals("OPTIONS") ) {
			doOptions(request,response);
		} else if (request.getMethod().equals("PUT") ) {
			doPut(request,response);
		} else if (request.getMethod().equals("TRACE") ) {
			doTrace(request,response);
		} else {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "Method \"" + request.getMethod() + "\" is not supported by this servlet");
		}
	}


	/**
	 * Frontend for calling service(HttpServletRequest,HttpServletResponse).
	 * <P>
	 * This method tries to typecast the ServletRequest and the
	 * ServletResponse to HttpServletRequest and HttpServletResponse
	 * and then call service(HttpServletRequest,HttpServletResponse).<BR>
	 *
	 * @param request The client's request
	 * @param response The class to write the response date to.
	 * @exception ServletException an error has occured
	 * @exception IOException an error has occured
	 */
	public void service(ServletRequest request,
				ServletResponse response) throws ServletException, IOException {
		try {
		 	service((HttpServletRequest)request,(HttpServletResponse)response);
		} catch (ClassCastException e) {
			throw new ServletException("This servlet only excepts http requests");
		}
	}

	/**
	 * This method is called on a "HEAD" request.
	 * This is currently implemented as a minimal HEAD implementation.<BR>
	 * The server returns the OK status, the date and the serverinfo.<BR>
	 *
	 * @exception ServletException if an Servlet Exception occurs
	 * @exception IOException if an IOException occurs
	 */
	protected void doHead(HttpServletRequest request,
				HttpServletResponse response) throws ServletException, IOException {

		if(! doHeadCallsDoGet) {
			response.setStatus(HttpServletResponse.SC_OK);
			response.setDateHeader("Date",System.currentTimeMillis());
			response.setHeader("Server",getServletConfig().getServletContext().getServerInfo());
		} else {
			doGet(request, new DoHeadHttpServletResponse(response));
		}
	}

	/**
	 * This class is built for use in the doHead method of HttpServlet.<BR>
	 * It enables the doHead method to have the doGet method do the work of
	 * a HEAD method without him noticing it.<BR>
	 * The only important method in this class is the getOutputStream
	 * method.
	 * This method returns a ServletOutputStream that has an empty
	 * write(int) and an empty write(byte[],int,int) implementation.
	 * <P>
	 * The real response is passed to this class in the constructor.
	 * All other methods call this response to do the work.
	 * This means that all those methods have an implementation of this
	 * form:
	 * <PRE>
	 *	void foo () {
	 *		realResponse.foo();
	 *	}
	 * </PRE>
	 */
	 private class DoHeadHttpServletResponse
	 	implements HttpServletResponse {

		HttpServletResponse		myResponse;


		DoHeadHttpServletResponse(HttpServletResponse response) {
			myResponse = response;
		}


		public void addCookie(Cookie aCookie) {
			myResponse.addCookie(aCookie);
		}


		public boolean containsHeader(String name) {
			return myResponse.containsHeader(name);
		}


	#ifdef VERSION_2_1

		public String encodeRedirectURL(String url) {
			return myResponse.encodeRedirectURL(url);
		}


		public String encodeURL(String url) {
			return myResponse.encodeURL(url);
		}

	#endif

		public void sendError(int errorCode, String errorMessage) throws IOException {
			myResponse.sendError(errorCode,errorMessage);
		}


		public void sendError(int errorCode) throws IOException {
			myResponse.sendError(errorCode);
		}


		public void sendRedirect(String location) throws IOException {
			myResponse.sendRedirect(location);
		}


		public void setDateHeader(String name, long value) {
			myResponse.setDateHeader(name,value);
		}


		public void setHeader(String name, String value) {
			myResponse.setHeader(name,value);
		}


		public void setIntHeader(String name, int value) {
			myResponse.setIntHeader(name,value);
		}


		public void setStatus(int sc) {
			myResponse.setStatus(sc);
		}


		/**
	#ifdef VERSION_2_1
		 * @deprecated
	#endif
		 */
		public void setStatus(int sc, String sm) {
			myResponse.setStatus(sc,sm);
		}


		
		/**
		#ifdef VERSION_2_1
		 * @deprecated
		#endif
		 */
		public String encodeUrl(String url) {
			return myResponse.encodeUrl(url);	
		}


		/**
		 #ifdef VERSION_2_1
		 * @deprecated
		 #endif
		 */
		public String encodeRedirectUrl(String url) {
			return myResponse.encodeUrl(url);	
		}


	/* ********************************************************************
	 *                                                                    *
	 *                  ServletResponse Methods                           *
	 *                                                                    *
	 **********************************************************************/

		public void setContentLength(int length) {
			myResponse.setContentLength(length);
		}
		
		public void setContentType(String type) {
			myResponse.setContentType(type);
		}


		/**
		 * Returns a dummy ServletOutputStream.
		 * This method is the only one whose implementation differs from that
		 * of an ordinary HttpServletResponse.
		 * The ServletOutputStream it returns has an empty write(int) and an empty write(byte[],int,int) implementation.
		 */
		public ServletOutputStream getOutputStream() throws IOException {
			return new ServletOutputStream() {
				public void write(int aByte)
							throws IOException {
				}

				public void write(byte[] buffer,
							int offset,
							int length)
							throws IOException {
				}
			};
		}


		public PrintWriter getWriter() throws IOException {
			return myResponse.getWriter();
		}


		public String getCharacterEncoding() {
			return myResponse.getCharacterEncoding();
		}
	}
}
