/*
 * UnavailableException.java -- Exception indicating that this servlet is
 *                              unavailable
 *
 * Copyright (c) 1998, 1999 by Free Software Foundation, Inc.
 * Written by Paul Siegmann (pauls@euronet.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation, version 2. (see COPYING.LIB)
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA
 */

package javax.servlet;

/**
 * This is a special kind of exception telling the server that this particular
 * servlet is currently not available.
 * It has two kinds of unavailability:
 * <DL>
 * <DT>Permanent unavailable
 * <DD>This servlet is now and forever in the future unavailable.
 * If another class in spite of this fact asks this exception for how long it 
 * is unavailable it returns a negative number of seconds.
 * (-1 as a matter of fact)
 * <DT>Temporary unavailable
 * The servlet is currently unavailable, but will be available within
 * a certain number of seconds.
 * A class can ask the exception for that number of seconds.
 * </DL>
 */
public class UnavailableException
	extends ServletException 
{
	private Servlet	myServlet;
	private boolean	myPermanent;
	private int	mySeconds;

	/**
	 * Constructor for a permanent unavailable exception
	 */
	public UnavailableException(Servlet servlet, String message) {
		super(message);
		myServlet = servlet;
		myPermanent = true;
	}


	/**
	 * Constructor for a temporary unavailable exception
	 */
	public UnavailableException(int seconds, Servlet servlet, String message) {
		super(message);
		myServlet = servlet;
		mySeconds = seconds;
		myPermanent = false;
	}


	/**
	 * Check whether the servlet is permanently unavailable
	 *
	 * @return whether the servlet is permanently unavailable
	 */
	public boolean isPermanent() {
		return myPermanent;
	}


	/**
	 * Gets the servlet that is unavailable
	 */
	public Servlet getServlet() {
		return myServlet;
	}

	/**
	 * Gets the number of seconds the servlet is unavailable
	 *
	 * @return the number of seconds. Negative if permanently unavailable
	 */
	public int getUnavailableSeconds() {
		if(! myPermanent) {
			return mySeconds;
		} else {
			return -1;
		}
	}
}
